<?php

namespace app\core\backend\controllers;

use Yii;
use app\core\models\Gitlab;
use yii\data\ActiveDataProvider;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use app\core\models\Project;
use app\core\models\Issue;
use app\core\models\IssueComment;
use app\core\models\AppUser as User;
use app\gitlab\models\Event;
use app\impulse\models\Log;

/**
 * GitlabController implements the CRUD actions for Gitlab model.
 */
class GitlabController extends Controller
{
    
    public function actionIndex()
    {
        $model = $this->findModel();

        if ($model->load(Yii::$app->request->post()) && $model->save()) {
            Yii::$app->cache->flush();
            return $this->redirect(['/']);
        } else {
            return $this->render('update', [
                'model' => $model,
            ]);
        }
    }

    protected function findModel()
    {
        if (($model = Gitlab::find()->one()) !== null) {
            return $model;
        } else {
            $model = new Gitlab();
            $model->save();
            return $model;
        }
    }

    public function actionClear($confirm = null)
    {
        if (!$confirm) {
            return $this->render('_clear');
        }

        $log = '';

        $projects = Project::find()->where([
            'not', ['gitlab_id' => null],
        ])->all();

        foreach ($projects as $project) {
            $log .= 'Deleting project ' . $project->gitlab_id . "<br>";
            Yii::$app->gitlab->raw('/projects/' . $project->gitlab_id, [], [], false, 'DELETE');

            $issues = Issue::find()->where([
                'project_id' => $project->id
            ])->all();

            foreach ($issues as $issue) {
                $comments = IssueComment::find()->where([
                    'issue_id' => $issue->id
                ])->all();

                foreach ($comments as $comment) {
                    $comment->delete();
                }

                $issue->delete();
            }

            $project->delete();
        }

        $users = User::find()->where([
            'not', ['gitlab_id' => null]
        ])->all();

        foreach ($users as $user) {
            $log .= 'Deleting user ' . $user->gitlab_id . "<br>";
            Yii::$app->gitlab->raw('/users/' . $user->gitlab_id, [
                'hard_delete' => 'true',
            ], [], false, 'DELETE');

            $user->delete();
        }

        foreach (Event::find()->all() as $event) {
            $log .= 'Deleting event ' . $event->id . "<br>";
            $event->delete();
        }

        $log .= 'Deleting Impulse records <br>';
        foreach (Log::find()->all() as $record) {
            $record->delete();
        }

        $log .= 'Done <br>';

        return $this->render('_clear', [
            'log' => $log,
        ]);
    }

    public function actionUsersSearch($q = null)
    {
        Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
        $users = [];

        if ($q) {
            $data = Yii::$app->gitlab->raw('search', [
                'scope' => 'users',
                'search' => $q,
            ]);
            foreach ($data as $item) {
                $users[] = [ 'id' => $item->id, 'text' => $item->name .' (' . $item->username . ')' ];
            }

            return ['results' => $users];
        }

        $ret = [];
        $pager = new \Gitlab\ResultPager(Yii::$app->gitlab->client);
        $gUsers = $pager->fetch(Yii::$app->gitlab->client->api('users'), 'all');

        foreach ($gUsers as $gUser) {
            $ret[] = [ 'id' => $gUser['id'], 'text' => $gUser['name'] .' (' . $gUser['username'] . ')' ];
        }

        return ['results' => $ret];
    }
}
