<?php

namespace app\core\backend\controllers;

use Exception;
use Gitlab\ResultPager;
use Yii;
use yii\web\Controller;
use yii\helpers\ArrayHelper;
use app\planfix\helpers\ApiHelper;
use app\core\models\Project;
use app\planfix\classes\PFApi;
use app\core\jobs\MapProject;
use app\core\jobs\ImportJob;

class ProjectController extends Controller
{
   
    public function actionIndex($import = false)
    {
        $cache = Yii::$app->cache;
        $gitlabProjects = [];
        $planfixProjects = [];
        $error = null;

        try {
            if (Yii::$app->gitlab->client) {
                $gitlabProjects = $cache->getOrSet('gl-projects', function () {
                    $pager = new ResultPager(Yii::$app->gitlab->client);
                    $gProjects = $pager->fetchAll(Yii::$app->gitlab->client->api('projects'), 'all', Yii::$app->gitlab->isAdmin? []: [
                        ['membership' => true],
                    ]);

                    $ret = [];
                    foreach ($gProjects as $project) {
                        $ret[$project['id']] = $project['name'] . ' (' . $project['id'] . ')' ;
                    }
                    return $ret;
                }, 300);
            }
            

            $pProjects = $cache->getOrSet('pf-projects', function () {
                return ApiHelper::getProjects();
            }, 300);
            $planfixProjects = ArrayHelper::map($pProjects, 'id', 'title');
        } catch (Exception $e) {
            $error = $e;
        }
        return $this->render($import? 'import' : 'index', [
            'gitlabProjects' => $gitlabProjects,
            'planfixProjects' => $planfixProjects,
            'error' => $error,
        ]);
    }

    public function actionSync()
    {
        $gitlabId = Yii::$app->request->post('project_gitlab');
        $planfixId = Yii::$app->request->post('project_planfix');

        if (!$gitlabId || !$planfixId) {
            return $this->redirect(['index']);
        }

        $project = Project::find()->where([
            'or',
            [ 'gitlab_id' => $gitlabId ],
            [ 'planfix_id' => $planfixId ]
        ])->one();

        if ($project) {
            $project->gitlab_id = $gitlabId;
            $project->planfix_id = $planfixId;
            $project->save();

            PFApi::syncPlanfixProject($planfixId, $gitlabId);
        } else {
            PFApi::syncPlanfixProject($planfixId, $gitlabId);
            $project = Project::find()->where([
                'planfix_id' => $planfixId,
                'gitlab_id' => $gitlabId,
            ])->one();

            if (!$project) {
                throw new Exception("Project sync failed");
            }
        }

        $syncIssues = Yii::$app->request->post('syncTasksGitlab');
        $syncTasks = Yii::$app->request->post('syncTasksPlanfix');

        Yii::$app->queue->push(new MapProject([
            'projectId' => $project->id,
            'syncIssues' => $syncIssues,
            'syncTasks' => $syncTasks
        ]));

        return $this->render('sync');
    }

    public function actionImport()
    {
        $pfProjects = Yii::$app->request->post('pf-projects', []);
        $glProjects = Yii::$app->request->post('gl-projects', []);

        $pfProjectTasks = Yii::$app->request->post('pf-projects-tasks', []);
        $glProjectTasks = Yii::$app->request->post('gl-projects-tasks', []);


        Yii::$app->queue->push(new ImportJob([
            'pfProjects' => $pfProjects,
            'glProjects' => $glProjects,
            'pfProjectTasks' => $pfProjectTasks,
            'glProjectTasks' => $glProjectTasks,
        ]));

        return $this->render('sync');
    }
}
