<?php

namespace app\core\backend\controllers;

use Exception;
use Gitlab\ResultPager as ResultPagerAlias;
use Yii;
use yii\web\Controller;
use app\planfix\helpers\ApiHelper;
use app\planfix\classes\PFApi;
use app\core\models\AppUser as User;

class UserController extends Controller
{
   
    public function actionIndex($contacts = false)
    {
        $cache = Yii::$app->cache;
        $gitlabUsers = [];
        $planfixUsers = [];
        $error = null;

        try {
            if (Yii::$app->gitlab->client) {
                $gitlabUsers = $cache->getOrSet('gl-users', function () {
                    $ret = [];
                    $pager = new ResultPagerAlias(Yii::$app->gitlab->client);
                    $gUsers = $pager->fetch(Yii::$app->gitlab->client->api('users'), 'all');

                    foreach ($gUsers as $gUser) {
                        $ret[$gUser['id']] = $gUser['name'] .' (' . $gUser['username'] . ')';
                    }
                    return $ret;
                }, 300);
            }

            $pUsers = $cache->getOrSet('pf-users', function () {
                return ApiHelper::getUsers();
            }, 300);
            foreach ($pUsers as $user) {
                $planfixUsers[$user['id']] = $user['name'] . ' ' . (is_array($user['lastName']) ? '' : $user['lastName']);
            }
            
            if ($contacts) {
                $pUsers = ApiHelper::getContacts();
                foreach ($pUsers as $user) {
                    $planfixUsers[$user['userid']] = $user['name'] . ' ' . (is_array($user['lastName']) ? '' : $user['lastName']);
                }
            }
        } catch (Exception $e) {
            $error = $e;
        }
        return $this->render('index', [
            'gitlabUsers' => $gitlabUsers,
            'planfixUsers' => $planfixUsers,
            'error' => $error,
        ]);
    }

    public function actionSync()
    {
        $gitlabId = Yii::$app->request->post('user_gitlab');
        $planfixId = Yii::$app->request->post('user_planfix');

        if (!$gitlabId || !$planfixId) {
            return $this->redirect(['index']);
        }

        $userQuery = User::find()->where([
            'or',
            [ 'gitlab_id' => $gitlabId ],
            [ 'planfix_id' => $planfixId ]
        ]);

        $users = $userQuery->all();

        foreach ($users as $user) {
            $user->delete();
        }
        
        PFApi::getUserFromPlanfix($planfixId, true, $gitlabId);
        
        
        return $this->render('sync');
    }
}
