<?php

namespace app\core\backend\controllers;

use Yii;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\web\ServerErrorHttpException;
use yii\filters\VerbFilter;
use app\impulse\models\Log;
use yii\data\ActiveDataProvider;
use app\core\models\Queue;
use yii\helpers\Url;
use app\core\helpers\UpdateHelper;

class WelcomeController extends Controller
{
    public function behaviors()
    {
        return [
            'verbs' => [
                'class' => VerbFilter::className(),
                'actions' => [
                    'clear-log' => ['POST'],
                    'perform-update' => ['POST'],
                ],
            ],
        ];
    }

    public function actionIndex()
    {
        $errors = Log::find()->where([
            'category' => 'app_error'
        ]);

        $dataProvider = new ActiveDataProvider([
            'query' => $errors,
            'sort'=> ['defaultOrder' => ['id' => SORT_DESC]]
        ]);

        $queueDataProvider = new ActiveDataProvider([
            'query' => Queue::find(),
            'sort'=> ['defaultOrder' => ['id' => SORT_DESC]]
        ]);

        return $this->render('index', [
            'dataProvider' => $dataProvider,
            'queueDataProvider' => $queueDataProvider,
        ]);
    }

    public function actionSettings()
    {
        return $this->render('settings');
    }

    public function actionQueue()
    {
        $queueDataProvider = new ActiveDataProvider([
            'query' => Queue::find(),
            'sort'=> ['defaultOrder' => ['id' => SORT_DESC]]
        ]);

        return $this->render('queue', [
            'queueDataProvider' => $queueDataProvider,
        ]);
    }

    public function actionClearLog()
    {
        Yii::$app->db->createCommand()->truncateTable('impulse_log')->execute();

        return $this->redirect(['/']);
    }

    public function actionMail()
    {
        $message = Yii::$app->request->post('message');

        $send = Yii::$app->mailer->compose()->setHtmlBody($message)
         ->setReplyTo(Yii::$app->user->identity->email)
         ->setTo('mailbox@poravinternet.ru')
         ->setSubject('Обратная связь GitPf на ' . Url::base(''))
         ->send();

        if ($send) {
            Yii::$app->session->setFlash('success', 'Сообщение отправлено');

            return $this->redirect(['/']);
        }

        throw new ServerErrorHttpException("Ошибка при отправке сообщения");
    }

    public function actionClearUpdated()
    {
        Yii::$app->settings->set('app.core', 'updated-to', null);
        return $this->redirect(['index']);
    }

    public function actionPerformUpdate()
    {
        $result = UpdateHelper::performUpdate();
        if ($result) {
            Yii::$app->session->setFlash('success', 'Обновление выполнено успешно!');
        } else {
            Yii::$app->session->setFlash('danger', 'Ошибка при выполнении обновления. Попробуйте выполнить из консоли php yii update/updater/update');
        }
        return $this->redirect(['index']);
    }
}
