<?php

use platform\widgets\Card;
use yii\helpers\Html;
use app\core\assets\QueueAsset;
use yii\grid\GridView;
use yii\helpers\Url;
use app\core\helpers\UpdateHelper;

$this->title = 'Начальная страница';
$this->params['breadcrumbs'][] = $this->title;

QueueAsset::register($this);


$cache = Yii::$app->cache;

$haveUpdates = $cache->getOrSet('app-updates', function () {
    return \app\core\helpers\UpdateHelper::checkUpdates();
}, 60);

if ($haveUpdates) {
    $changelog = $cache->getOrSet('app-updates-' . $haveUpdates, function () use ($haveUpdates) {
        UpdateHelper::getUpdateChangeLog(null, $haveUpdates);
    }, 60);
}

$updatedTo = Yii::$app->settings->get('app.core', 'updated-to', null);
if ($updatedTo) {
    $history = '';
    $updatedFrom = Yii::$app->settings->get('app.core', 'updating-from', null);
    if ($updatedFrom) {
        $history = shell_exec("cd " . Yii::getAlias('@project') . "; git log --pretty=oneline --abbrev-commit " . $updatedTo . "..." . $updatedFrom);
    }
}
?>

<div>
    <?php if ($updatedTo) : ?>
        <?php Card::begin([
            'title' => 'Выполнено обновление',
        ]); ?>
            <p>
                Выполнено обновление до версии <?=$updatedTo?>
            </p>
            <?php if ($history) : ?>
                <pre><?=$history?></pre>
            <?php endif;?>
            <a class="btn btn-secondary" href="<?=Url::to(['clear-updated'])?>">ОК</a>
        <?php Card::end(); ?>
    <?php endif;?>
    <div class="row">
        <?php if ($haveUpdates) : ?>
        <div class="col-md-12 mb-2">
            <?php Card::begin([
                'title' => 'Доступно обновление',
            ]); ?>
                <p>
                    Доступно обновление до версии <?=$haveUpdates?>
                </p>
                <?php if ($changelog) : ?>
                    <pre><?=$changelog?></pre>
                <?php endif;?>
                <form action="<?=Url::to(['perform-update'])?>" method="POST">
                    <?=yii\helpers\Html::hiddenInput(Yii::$app->request->csrfParam, Yii::$app->request->csrfToken)?>
                    <button class="btn btn-secondary" type="submit">Выполнить</button>
                </form>
            <?php Card::end(); ?>
        </div>
        <?php endif;?>
        <div class="col-md-12 mb-2">
            <div class="docs-hint">
                <a href="/docs">Документация</a>
            </div>
        </div>
    </div>
    <div class="row">
        <div class="col-md-5">
            <?php Card::begin(['title' => 'GitPF']);?>

                <div class="row">
                    <div class="col-md-4">
                        <img class="rounded-circle" src="/logo/gitpf.png" style="width: 100%; margin-top: 10px;">
                    </div>                    
                    <div class="col-md-8">
                        <p>GitPF - версия <?=UpdateHelper::version()?></p>
                        <a href="<?=Url::to(['/core/welcome/settings'])?>"><button class="btn btn-primary">Настроить</button></a>
                    </div>
                </div>
                

                
            <?php Card::end();?>

            <?php Card::begin(); ?>
            <h4>Очередь задач</h4>
            <?=GridView::widget([
                'dataProvider' => $queueDataProvider,
                'columns' => [
                    ['class' => 'yii\grid\SerialColumn'],

                    'job',
                    'reserved_at',
                    'pushed_at',
                    'done_at',
                 
                ]
            ])?>
            <?php Card::end(); ?>
        </div>
        <div class="col-md-7">
            <?=$this->render('_requests.php')?>
        </div>
    </div>
    
</div>
<div class="row">
    <div class="col-md-12">
        <?php Card::begin(); ?>
        <h4>Ошибки приложения</h4>
        <a href="<?=Url::to(['/core/welcome/clear-log'])?>" class="btn btn-danger mb-2" data-method="post">Очистить лог</a>
        <?=GridView::widget([
            'dataProvider' => $dataProvider,
            'columns' => [
                ['class' => 'yii\grid\SerialColumn'],

                'message',
                'created_at',
             
            ]
        ])?>
        <?php Card::end(); ?>
    </div>



</div>


<script type="text/javascript">
    document.addEventListener('DOMContentLoaded', function(){
        setInterval(function(){
            $.get('/backend/core/welcome/queue', function(data) {
                $('#w0').html($(data).find('#w0').html())
            })
        }, 1000)
        
    })
</script>
<style type="text/css">
    .card-body {
        overflow: auto;
    }
</style>