<?php

use platform\widgets\Card;
use yii\helpers\Html;
use app\core\assets\QueueAsset;
use yii\grid\GridView;
use yii\helpers\Url;
use app\core\helpers\UpdateHelper;

$this->title = 'Настройки';
$this->params['breadcrumbs'][] = $this->title;

QueueAsset::register($this);


$cache = Yii::$app->cache;

$stages = [
    'planfix' => [
        'title' => 'Соединение с planfix',
        'url' => ['/core/planfix/index'],
        'complete' => function () use ($cache) {
            if (!Yii::$app->planfix->account) {
                return false;
            }

            if (!Yii::$app->planfix->api_key) {
                return false;
            }

            if (!Yii::$app->planfix->api_secret) {
                return false;
            }

            if (!Yii::$app->planfix->tpl_dev_id) {
                return 'Не задан шаблон задачи';
            }

            if (!Yii::$app->planfix->pr_field_lead_dev) {
                return 'Не задано поле: ведущий разработчик';
            }

            if (!Yii::$app->planfix->time_analytic_id) {
                return 'Не задаиа аналитика: Фактическое время';
            }

            if (!Yii::$app->planfix->time_analytic_date_id) {
                return 'Не задано поле аналитики: Дата';
            }

            if (!Yii::$app->planfix->time_analytic_time_id) {
                return 'Не задано поле аналитики: Время';
            }

            if (!Yii::$app->planfix->process_id) {
                return 'Не задан процесс разработки';
            }

            try {
                $processes = $cache->getOrSet('pf-status', function () {
                    return Yii::$app->planfix->getProcessesList();
                }, 600);
            } catch (\Exception $e) {
                return 'Ошибка подключения: ' . $e->getMessage();
            }

            return true;
        }
    ],
    'gitlab' => [
        'title' => 'Соединение с GitLab',
        'url' => ['/core/gitlab/index'],
        'complete' => function () use ($cache) {
            
            if (!Yii::$app->gitlab->token) {
                return false;
            }

            if (!Yii::$app->gitlab->url) {
                return false;
            }

            try {
                $gProjects = $cache->getOrSet('gitlab-status', function () {
                    return Yii::$app->gitlab->client->projects()->all();
                }, 600);
            } catch (\Throwable $e) {
                return 'Ошибка подключения: ' . $e->getMessage();
            }

            return true;
        }
    ],
    'users' => [
        'title' => 'Соответствие пользователей',
        'url' => ['/core/user-list/index'],
        'complete' => function () {
            return false;
        },
        'comment' => 'Задание соответствия пользователей Planfix и GitLab. Используйте, чтобы сопоставить существующих пользователей'
    ],
    'projects' => [
        'title' => 'Соответствие проектов',
        'url' => ['/core/project-list/index'],
        'complete' => function () {
            return false;
        },
        'comment' => 'Задание соответствия проектов Planfix и GitLab. Используйте, чтобы сопоставить существующие проекты, их статусы'
    ],
    'cleanup' => [
        'title' => 'Очистка данных',
        'url' => ['/core/gitlab/clear'],
        'danger' => true,
        'complete' => function () {
            return false;
        },
        'comment' => 'Полное и <b>НЕОБРАТИМОЕ</b> удаление всех синхронизированных данных (проекты, задачи, пользователи) из GitLab'
    ]
];

?>

<div class="row">
    <?php foreach ($stages as $key => $stage) :?>
        <?php
            $complete = $stage['complete']();
        ?>
        <div class="col-md-6">
        <?php Card::begin([
            'title' => $stage['title'] . '<div style="float: right;">' . ($complete === true? '<i style="color: green;" class="fa fa-check-circle"></i>' : Html::a('Перейти <i class="fa fa-arrow-circle-right"></i>', $stage['url'], ['class' => isset($stage['danger'])? 'btn btn-danger' : 'btn btn-secondary', 'style' => 'display: inline;']) ) . '</div>'
        ]); ?>            

        <?php if ($complete) : ?>
            <div style="float: right;">
                <?=Html::a('Изменить <i class="fa fa-arrow-circle-right"></i>', $stage['url'], ['class' => 'btn btn-secondary', 'style' => 'display: inline;'])?>
            </div>
        <?php endif;?>            


        <?php if ($complete && $complete !== true) : ?>
            <div class="alert alert-danger" role="alert">
                <?=$complete?>
            </div>
        <?php endif; ?>

        <?php if (isset($stage['comment'])) : ?>
            <div class="alert alert-secondary" role="alert">
                <?=$stage['comment']?>
            </div>
        <?php endif; ?>
        <?php Card::end();?>
    </div>
    <?php endforeach; ?>
</div>