<?php

namespace app\core\frontend\controllers;

use Yii;
use yii\web\Controller;
use yii\web\NotFoundHttpException;
use yii\filters\VerbFilter;
use platform\models\User;
use yii\web\ForbiddenHttpException;
use app\core\helpers\UpdateHelper;

class IndexController extends Controller
{

    public function behaviors()
    {
        return [
            'access' => [
                'class' => \yii\filters\AccessControl::className(),
                'only' => ['package'],
                'rules' => [
                    [
                        'allow' => true,
                        'roles' => ['admin'],
                    ],
                    // everything else is denied
                ],
            ],
        ];
    }
    public function actionIndex()
    {
        return $this->render('index');
    }

    public function actionDocs()
    {
        return $this->render('docs');
    }

    public function actionFaq()
    {
        return $this->render('faq');
    }

    public function actionCreateAdmin()
    {

        $username = Yii::$app->request->post('username');
        $email = Yii::$app->request->post('email');
        $password = Yii::$app->request->post('password');

        if (!$username || !$password || !$email) {
            return $this->render('create-admin');
        }

        if (User::find()->count()) {
            throw new ForbiddenHttpException("Admin already created!");
        }

        $user = Yii::$app->userManager->createUser(
            $username,
            $email,
            $password
        );

        if (null !== $user) {
            $this->assign($user->id, 'admin');
            

            return $this->render('create-admin', [
                'complete' => 1,
            ]);
        } else {
            throw new Exception("Error Processing Request");
            return $this->render('create-admin');
        }
    }

    public function actionVersion()
    {
        Yii::$app->response->format = \yii\web\Response::FORMAT_JSON;
        $version = Yii::$app->settings->get('app.core', 'last-version', null);

        return [
            'version' => $version,
        ];
    }

    public function actionVersionHistory($from, $to = null)
    {
        if (!$to) {
            $to = 'master';
        }

        return UpdateHelper::getHistory($from, $to);
    }

    public function actionPackage()
    {
        $version = Yii::$app->settings->get('app.core', 'last-version', null);
        if (!$version) {
            throw new NotFoundHttpException("Package not found");
        }
        return $this->redirect("/static/packages/gitpf-$version.zip");
    }

    /**
     * @param $id
     * @param $role
     */
    private function assign($id, $role)
    {
        $auth = Yii::$app->authManager;

        $role = $auth->getRole($role);
        $auth->assign($role, $id);
    }
}
