<?php

use app\core\helpers\UpdateHelper;

$this->title = 'Документация GitPF';

?>
<div class="docs" style="text-align: left;">
    <h3 id="installation">Установка</h3>
    <p>
        Систему можно установить несколькими сособами:
        <ul>
            <li>Используя скрипт для автоматической установки (docker / nginx + php-fpm)</li>
            <li>Установка вручную (docker)</li>
            <li>Установка вручную (nginx + php-fpm)</li>
        </ul>
    </p>

    <h4>Установка с использованием скрипта:</h4>
    <p>
        Рекомендуется использовать на чистом сервере с Debian / Ubuntu / CentOS.<br>        
    </p>
    <p style="color: #909029">        
        Выполните на сервере:
        <pre>curl https://gitpf.poravinternet.ru/install.sh --output install.sh; bash install.sh <?=UpdateHelper::version();?></pre>
    </p>
    <p>
        Затем ответьте на вопросы установщика. После установки, перейдите на http://{адрес сервера}/setup<br>
        Введите настройки базы данных, после чего будут выполнены миграции БД.
    </p>

    <h4>Ручная установка: </h4>
    <ul>
        <li>Скачать установочный пакет по <a href="https://gitpf.poravinternet.ru/package">ссылке</a></li>
        <li>Распаковать на сервере</li>        
    </ul>    
    <p style="color: #909029">
        Если вы используете docker, вы можете запустить сервис используя docker-compose
        <pre>
            cp .env.example .env
            # Отредактируйте настройки MYSQL в .env файле
            bash docker.sh
        </pre>
        <p>Или:</p>
        <pre>
            cp .env.example .env
            # Отредактируйте настройки MYSQL в .env файле
            
            git reset --hard
            docker-compose up -d
        </pre>
    </p>
    <p>Корень веб-сервера должен смотреть в папку web из архива</p>

    <p>Пример конфигурации для Nginx</p>
    <pre>
server {
    listen 80;
    server_name gitpf.poravinternet.ru;
    return 301 https://$host$request_uri;
}

server {
    listen 443 ssl;
    server_name   gitpf.poravinternet.ru;

    client_max_body_size 128M;
    charset utf-8;

    root /srv/gitpf/www/web;
    index index.php;

    access_log  /var/log/nginx/gitpf-access.log;
    error_log   /var/log/nginx/gitpf-error.log;

    location / {
        try_files $uri $uri/ /index.php?$args;
    }

    location /backend {
        try_files $uri $uri/ /backend/index.php?$args;
    }

    location ~ ^/assets/.*\.php$ {
        deny all;
    }

    location ~ \.php$ {
        include fastcgi_params;
        fastcgi_param SCRIPT_FILENAME $document_root$fastcgi_script_name;
        fastcgi_pass 127.0.0.1;
        try_files $uri =404;
    }

    ssl_certificate /etc/letsencrypt/live/gitpf.poravinternet.ru/fullchain.pem; # managed by Certbot
    ssl_certificate_key /etc/letsencrypt/live/gitpf.poravinternet.ru/privkey.pem; # managed by Certbot
}

        </pre>
        <p>
            Затем перейти по адресу http(s)://yourdomain.sth/setup и следовать инструкциям установщика.
        </p>
        <p>
            Установщик запросит доступ к БД mysql и запустит миграции. 
        </p>

        <h4 id="queue">Настройка очереди</h4>
        <p style="color: #909029">При использовании скрипта установки или docker контейнера, очередь настраивается автоматически</p>
        <p>Для работы, GitPF использует очередь. Для настройки очереди необходимо добавить в systemd unit:</p>
        <pre>
[Unit]
Description=GitPf queue
After=network.target

[Service]
Type=simple
User=root
ExecStart=/usr/bin/php /srv/gitpf/www/yii queue/listen
Restart=on-failure

[Install]
WantedBy=multi-user.target
        </pre>
        <p>Затем systenctl daemon-reload и systemctl enable gitpf_queue</p>

        <h4 id="cron">Настройка заданий cron</h4>
        <p style="color: #909029">При использовании скрипта установки или docker контейнера, задания cron настраиваются автоматически</p>
        <p>
            Для выполнения служебных заданий используется крон. Для настройки, достаточно выполнить crontab -e от пользователя, от имени которого выполняется php и добавить следующую строку:
        </p>

        <pre>
            * * * * * php /srv/gitpf/www/yii schedule/run
        </pre>

        <p>
            На этом этап установки можно считать завершенным
        </p>
    
    <h3 id="configuration">Настройка</h3>
        <p>Настройка GitPF сводится к следующим этапам:</p>
        <ul>
            
            <li>Настройка подключения к Gitlab</li>
            <li>Установка конфигурации в Планфикс</li>
            <li>Настройка подключения к Planfix</li>
            <li>Сопоставление пользователей</li>
            <li>Сопоставление проектов</li>


        </ul>

        <h4 id="gitlab-connection">Настройка подключения к Gitlab</h4>
        <p>
            На <a href="/backend/core/welcome/settings">странице</a> нажмите на кнопку 'Выполнить' под Соединение с GitLab или в меню Сервис -> GitLab (<a href="/backend/core/gitlab/index">ссылка</a>)
        </p>
        <p>
            В поле Token введите токен доступа администратора GitLab, который можно получить по адресу https://gitlab/profile/personal_access_tokens
        </p>
        <p>
            При создании токена доступа, отметьте галочками разрешения sudo, api, read_user
        </p>
        <p>
            В поле Url введите адрес gitlab (https://gitlab.poravinternet.ru/)
        </p>

        <h4 id="planfix-connection">Настройка подключения к Planfix</h4>
        <p>
            В поле account ввести название аккаунта Планфикс (poravinternet для poravinternet.planfix.ru)
        </p>
        <p>
            В поле Api Key ввести аутентификационный ключ из (https://<account>.planfix.ru/?action=api)
        </p>
        <p>
            В поле Api Secret ввести токен авторизации Администратора Planfix. Создать можно на (https://<account>.planfix.ru/?action=api)
        </p>
        <p>
            Затем выбрать шаблон задачи на разработку (используется для создания новых задач в Планфикс из Gitlab)
        </p>
        <p>
            Выбрать Поле "Ведущий программист"
        </p>
        <p>
            Выбрать Процесс - используется для получения набора статусов из Планфикс
        </p>
        <p>
            Выбрать Аналитику Фактического времени, а затем ее поля. Поле сотрудник опционально (USERLIST).
        </p>

        <p style="background: darkgrey;padding: 5px;">
Перед подключением необходимо установить конфигурацию (<a href="https://planfix.ru/configurations/git-for-planfix-p56">https://planfix.ru/configurations/git-for-planfix-p56</a>).<br>
После установки конфигурации, необходимо создать новый проект, в котором выбираем шаблон Проект на разработку.<br>
В созданном проекте на разработку необходимо, в поле Роли в проекте, назначить Ведущего разработчика.<br>
Без данного назначения, невозможно выбрать параметр "Ведущий разработчки" при настройки подклчения к ПФ.<br>
После этого создаем задачу, в данном проекте, все необходимые параметры для подключения будут настроены.<br>
<br>
На новых аккаунтах после внедрения в Планфикс сущности Объект, после установки конфигурации, необходимо будет создать новый шаблон задачи, в котором: указываем проект - Проект на разработку, также, в дополнительных параметрах, необходимо поменять процесс со Стандартного на Разработка.<br>
Далее, создаем в Планфикс новый проект, выбираем шаблон Проект на разработку. В разделе Роли в проекте, назначаем ведущего программиста.<br>
После этого, создаем задачу в данном проекте. При создании задачи, меняем Стандартный шаблон, на Старый шаблон, в котором выбираем вновь созданный шаблон, для данного проекта.<br>
Остается только указать название вновь созданной задачи, все остальные параметры для подключения будут настроены.<br>
        </p>

        <h4 id="user-mapping">Настройка соответствия пользователей</h4>
        <p>
            Для корректной работы необходимо сопоставить пользователей в системах, если установка не была произведена с нуля и в Gitlab присутствуют сотрудники и проекты. В случае, если пропустить данный этап, пользователи в GitLab будут созданы автоматически используя данные из Планфикс.
        </p>
        <p>
            Для сопоставления нажмите на кнопку Выполнить под Соответствие пользователей на главной (Сервис -> GitPF) или по <a href="/backend/core/user-list/index">ссылке</a> и нажмите на кнопку Добавить. В Выпадающих полях выберите пользователей и нажмите Сопоставить.
        </p>

        <h4 id="project-mapping">Настройка соответствия проектов</h4>
        <p>
            Для корректной работы необходимо сопоставить проекты в системах, если установка не была произведена с нуля и в Gitlab присутствуют  и проекты. В случае, если пропустить данный этап, проекты в GitLab будут созданы автоматически исходя из проектов в Планфикс
        </p>
        <p>
            Для сопоставления нажмите на кнопку Выполнить под Соответствие пользователей на главной (Сервис -> GitPF) или по <a href="/backend/core/project-list/index">ссылке</a> и нажмите на кнопку Добавить. В Выпадающих полях выберите проекты и нажмите Сопоставить. Если необходимо перенести задачи, то выберите соответствующие флажки. Процесс переноса задач будет выполнен в фоне.
        </p>
        <p style="color: #909029">Для импорта проктов из одной системы в другую используйте Импорт на <a href="/backend/core/project-list/index">странице</a> проектов</p>
        <p style="color: #909029">Для сопоставления статусов используйте кнопку Настроить в колонке Сопоставление статусов на <a href="/backend/core/project-list/index">странице</a> проектов</p>
    <h3 id="usage">Использование</h3>
    <p>
        После завершения установки и настройки, комментарии и задачи будут синхронизироваться. Ддя того, чтобы задача из Планфикс попала в Обсуждения в Gitlab нужно указать шаблон задачи - задача на разработку из установленной конфигурации.
    </p>
    <p>
        Все новые задачи в Gitlab из сопоставленных проектов будут автоматически создаваться в Планфикс.
    </p>
    <p>
        Для отметки фактического времени в Gitlab добавьте комментарий с содержимым /spend 5h 10m
    </p>
    <p>
        В случае возникновения вопросов или ошибок, обращайтесь на <a href="mailto:mailbox@poravinternet.ru">mailbox@poravinternet.ru</a>
    </p>
    <p>
        Документация <a href="/static/docs/index.html">phpDocumentator</a>
    </p>
</div>

<style type="text/css">
    pre {
        background: darkgrey;
        padding: 5px;
    }
</style>