<?php

namespace app\core\helpers;

class TimeHelper
{

    public static function human($time, $label = false)
    {
        $min = $time / 60;
        if ($min >= 60) {
            $postfix = 'ч';
            $hrs = floor($min / 60);
            $min2 = floor($min - ($hrs * 60));

            if ($min2==0) {
                return $hrs . 'ч';
            } else {
                return $hrs . 'ч' . $min2 . 'м';
            }
        } else {
            return $min . 'м';
        }
    }

    public static function minutes($time)
    {
        return number_format($time / 60, 2);
    }

    public static function asDuration($time)
    {
        if ($time > 3600) {
            $time = (int)($time / 60) * 60;
        }

        if ($time > 3600 * 24) {
            $time = (int)($time / 3600) * 3600;
        }

        if ($time > 3600 * 24 * 30) {
            $time = (int)($time / (3600 * 24)) * 3600 * 24;
        }

        return \Yii::$app->formatter->asDuration($time);
    }

    public static function byDate($query, $dateAttribute, $valueAttribute, $count = false, $from = null, $to = null, $days = 10, $reverse = true)
    {
        $recordsByDate = [];



        if (!$to) {
            $to = date('Y-m-d 23:59:59');
        }


        if ($from || $days) {
            if (!$from) {
                $from = date('Y-m-d', strtotime('-' . $days . ' day'));
            }

            $query->andWhere([
                '>=', $dateAttribute, $from,
            ]);
        }

        $query = $query->andWhere([
            '<=', $dateAttribute, $to,
        ])->all();

        if ($days) {
            for ($i=0; $i < $days; $i++) {
                $recordsByDate[date('Y-m-d 00:00:00', strtotime('-' . $i . ' day'))] = 0;
            }
        }

        foreach ($query as $item) {
            $dateKey = date('Y-m-d 00:00:00', strtotime($item->{$dateAttribute}));

            if ($days && isset($recordsByDate[$dateKey])) {
                $amount = $count ? 1 : $item->{$valueAttribute};

                $recordsByDate[$dateKey] += $amount;
            } elseif (!$days) {
                if (!isset($recordsByDate[$dateKey])) {
                    $recordsByDate[$dateKey]  = 0;
                }
                
                $amount = $count? 1 : $item->{$valueAttribute};

                $recordsByDate[$dateKey] += $amount;
            }
        }
        if ($reverse) {
            krsort($recordsByDate);
        } else {
            ksort($recordsByDate);
        }

        foreach ($recordsByDate as $date => $time) {
            $recordsByDate[$date] = round($time, 1);
        }
        
        return $recordsByDate;
    }
}
