<?php
namespace app\core\jobs;

use Exception;
use Gitlab\ResultPager;
use Yii;
use app\planfix\classes\PFApi;
use yii\base\BaseObject;
use app\planfix\helpers\ApiHelper;
use app\core\models\QueueProgress;
use yii\queue\JobInterface;

class ImportJob extends BaseObject implements JobInterface
{
    public $pfProjects;
    public $glProjects;

    public $pfProjectTasks;
    public $glProjectTasks;
    public $progressModel = null;
    public $cStatus = '';

    public function execute($queue)
    {
        try {
            $pfProjects = $this->pfProjects;
            $glProjects = $this->glProjects;

            $pfProjectTasks = $this->pfProjectTasks;
            $glProjectTasks = $this->glProjectTasks;

            print_r($pfProjects);
            print_r($glProjects);
            print_r($pfProjectTasks);
            print_r($glProjectTasks);

            $this->writeProgress();

            $curr = 1;
            foreach ($pfProjects as $id => $value) {
                $this->writeProgress('Импорт проектов из Планфикс: ' . $curr . '/' . count($pfProjects), $this->map($curr, count($pfProjects)));
                PFApi::syncPlanfixProject($id);
                if (isset($pfProjectTasks[$id])) {
                    $tasks = ApiHelper::getTasks($id);
                    $tcurr = 1;
                    foreach ($tasks as $task) {
                        $this->writeProgress(
                            'Импорт задач: ' . $tcurr . '/' . count($tasks),
                            $this->map($tcurr, count($tasks)),
                            true
                        );
                        PFApi::syncIssuePlanfix($task['general'], true, true);
                        $tcurr++;
                    }
                }
                $curr++;
            }

            $curr = 1;
            foreach ($glProjects as $id => $value) {
                $this->writeProgress('Импорт проектов из GitLab: ' . $curr . '/' . count($glProjects), $this->map($curr, count($glProjects)));
                $project = Yii::$app->gitlab->importProject($id);

                if (isset($glProjectTasks[$id])) {
                    $pager = new ResultPager(Yii::$app->gitlab->client);
                    $issues = $pager->fetchAll(Yii::$app->gitlab->client->api('issues'), 'all', [$project->gitlab_id]);
                    $tcurr = 1;
                    foreach ($issues as $issue) {
                        $this->writeProgress(
                            'Импорт задач: ' . $tcurr . '/' . count($issues),
                            $this->map($tcurr, count($issues)),
                            true
                        );
                        Yii::$app->gitlab->importIssue($issue['iid'], $project->id, true, true);
                        $tcurr++;
                    }
                }
                $curr++;
            }

            $this->progressModel->delete();
        } catch (Exception $e) {
            Yii::$app->impulse->air('app_error', '[Ошибка]' . $e->getMessage() . ' on ' . $e->getLine() . 'line ' . $e->getFile());
            if ($this->progressModel) {
                $this->progressModel->cancel = 1;
                $this->progressModel->save();
            }
        }
    }

    public function writeProgress($status = null, $progress = 0, $append = false)
    {
        if (!$this->progressModel) {
            $qProgress = new QueueProgress();
            $qProgress->task_id = null;
            $qProgress->name = 'Импорт проектов';
            $this->cStatus = $qProgress->name;
            $qProgress->progress = 0;
            $qProgress->save();

            $this->progressModel = $qProgress;

            return $qProgress;
        }

        if ($append) {
            $this->progressModel->name = $this->cStatus . ' - ' . $status;
        } else {
            $this->progressModel->name = $status;
            $this->cStatus = $status;
        }
        
        $this->progressModel->progress = $progress;
        $this->progressModel->save();

        return true;
    }

    public function map($current, $total)
    {
        return ceil(100 * $current / $total);
    }
}
