<?php
namespace app\core\jobs;

use Exception;
use Gitlab\ResultPager;
use Yii;
use app\planfix\classes\PFApi;
use yii\base\BaseObject;
use app\planfix\helpers\ApiHelper;
use app\core\models\Project;
use app\core\models\QueueProgress;
use yii\queue\JobInterface;

class MapProject extends BaseObject implements JobInterface
{
    public $projectId = null;
    public $syncIssues = null;
    public $syncTasks = null;
    
    public function execute($queue)
    {
        try {
            $project = Project::find()->where(['id' => $this->projectId])->one();

            $qProgress = new QueueProgress();
            $qProgress->task_id = null;
            $qProgress->name = 'Сопоставление проекта: ' . $project->title;
            $qProgress->progress = 0;
            $qProgress->save();

            if ($this->syncIssues) {
                $pager = new ResultPager(Yii::$app->gitlab->client);
                $issues = $pager->fetchAll(Yii::$app->gitlab->client->api('issues'), 'all', [$project->gitlab_id]);
                
                $qProgress->name = 'Сопоставление проекта: ' . $project->title . ' - импорт задач из GitLab';
                $qProgress->save();

                foreach ($issues as $index => $issue) {
                    try {
                        Yii::$app->gitlab->importIssue($issue['iid'], $project->id, true, true);
                    } catch (Exception $e) {
                        $qProgress->name = '[Ошибка] Сопоставление проекта: ' . $project->title . " - импорт задач из Gitlab $index /" . count($issues) . ' ' . $e->getMessage() . ' ' . $e->getLine() . ' ' . $e->getFile();
                        $qProgress->cancel = 1;
                        $qProgress->save();

                        Yii::$app->impulse->air('app_error', $qProgress->name);


                        return true;
                    }
                    
                    $qProgress->name = 'Сопоставление проекта: ' . $project->title . " - импорт задач из Gitlab $index /" . count($issues);
                    $qProgress->progress = ceil($index * 100 / count($issues));
                    $qProgress->save();
                }
            }

            if ($this->syncTasks) {
                $tasks = ApiHelper::getTasks($project->planfix_id);

                $qProgress->name = 'Сопоставление проекта: ' . $project->title . ' - импорт задач из Planfix';
                $qProgress->save();

                foreach ($tasks as $index => $task) {
                    try {
                        PFApi::syncIssuePlanfix($task['general'], true, true);
                    } catch (Exception $e) {
                        $qProgress->name = '[Ошибка] Сопоставление проекта: ' . $project->title . " - импорт задач из Planfix $index /" . count($tasks) . ' ' . $e->getMessage() . ' on ' . $e->getLine() . 'line ' . $e->getFile();
                        $qProgress->cancel = 1;
                        $qProgress->save();

                        Yii::$app->impulse->air('app_error', $qProgress->name);

                        return true;
                    }
                    

                    $qProgress->name = 'Сопоставление проекта: ' . $project->title . " - импорт задач из Planfix $index /" . count($tasks);
                    $qProgress->progress = ceil($index * 100 / count($tasks));
                    $qProgress->save();
                }
            }

            $qProgress->delete();
        } catch (Exception $e) {
            if ($qProgress && $qProgress->id) {
                $qProgress->cancel = 1;
                $qProgress->title = '[Ошибка]' . $e->getMessage() . ' on ' . $e->getLine() . 'line ' . $e->getFile() . ' ' . $qProgress->title;
                $qProgress->save();

                 Yii::$app->impulse->air('app_error', $qProgress->name);
            } else {
                Yii::$app->impulse->air('app_error', '[Ошибка]' . $e->getMessage() . ' on ' . $e->getLine() . 'line ' . $e->getFile());
            }
        }

        return true;
    }
}
