<?php

namespace app\core\models;

use app\core\models\AppUser as User;
use yii\db\ActiveRecord;

/**
 * This is the model class for table "core_issue".
 *
 * @property int $id
 * @property int $gitlab_id
 * @property int $planfix_id
 * @property int $project_id
 * @property int $creator_id
 * @property string $description_gitlab
 * @property string $title
 * @property string $description
 * @property string $due_date
 * @property string $labels
 * @property string $status
 * @property int $assignee_id
 * @property string $created_at
 * @property User $creator
 * @property User $assignee
 */
class Issue extends ActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'core_issue';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['gitlab_id', 'planfix_id', 'assignee_id'], 'integer'],
            [['description', 'description_gitlab'], 'string'],
            [['due_date', 'created_at'], 'safe'],
            [['title', 'labels', 'status'], 'string', 'max' => 255],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'gitlab_id' => 'Gitlab ID',
            'planfix_id' => 'Planfix ID',
            'title' => 'Title',
            'description' => 'Description',
            'due_date' => 'Due Date',
            'labels' => 'Labels',
            'status' => 'Status',
            'assignee_id' => 'Assignee ID',
            'created_at' => 'Created At',
        ];
    }
    
    public function getCreator()
    {
        return $this->hasOne(User::class, ['id' => 'creator_id']);
    }

    public function getAssignee()
    {
        return $this->hasOne(User::class, ['id' => 'assignee_id']);
    }

    public function getProject()
    {
        return $this->hasOne(Project::class, ['id' => 'project_id']);
    }

    public function getComments()
    {
        return $this->hasMany(IssueComment::class, ['issue_id' => 'id']);
    }
}
