<?php

namespace app\core\models;

use Yii;
use app\core\models\AppUser as User;
use League\HTMLToMarkdown\HtmlConverter;
use yii\db\ActiveRecord;

/**
 * This is the model class for table "core_issue_comment".
 *
 * @property int $id
 * @property int $gitlab_id
 * @property int $issue_id
 * @property int $creator_id
 * @property int $is_log
 * @property string $text
 * @property string $created_at
 * @property string $origin Where comment came from
 * @property string $pf_time
 * @property string $pf_time_old
 * @property string $updated_at
 * @property int $planfix_id
 * @property string $text_gitlab Gitlab body
 * @property Issue $issue Linked Issue
 * @property User $creator User created Comment
 * @property string $files Comma separated files
 */
class IssueComment extends ActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'core_issue_comment';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['issue_id', 'creator_id'], 'required'],
            [['text', 'text_gitlab', 'pf_time', 'origin'], 'string'],
            [['issue_id', 'creator_id', 'is_log', 'gitlab_id', 'planfix_id'], 'integer'],
            [['created_at', 'updated_at'], 'safe'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'text' => 'Text',
            'issue_id' => 'Issue ID',
            'creator_id' => 'Creator ID',
            'project_id' => 'Project ID',
            'is_log' => 'Is Log',
            'created_at' => 'Создан',
            'updated_at' => 'Изменен',
            'planfix_id' => 'Planfix ID',
        ];
    }

    public function getIssue()
    {
        return $this->hasOne(Issue::class, ['id' => 'issue_id']);
    }

    public function getCreator()
    {
        return $this->hasOne(User::class, ['id' => 'creator_id']);
    }


    public function getGitlabText()
    {
        $markdown = new HtmlConverter();
        $text = $markdown->convert($this->text);

        $files = explode(',', $this->files);

        foreach ($files as $file) {
            if (!$file) {
                continue;
            }
            $text .= "\r\n<$file>   ";
        }

        $notify = explode(',', $this->notify_list);

        foreach ($notify as $notifyUser) {
            if (!$notifyUser) {
                continue;
            }
            $user = User::findOne($notifyUser);
            $gitlabUser = Yii::$app->gitlab->raw("users/$user->gitlab_id");
            if ($gitlabUser) {
                $text .= "\r\n@$gitlabUser->username   ";
            }
        }

        if (!Yii::$app->gitlab->isAdmin && $this->origin === 'planfix') {
            $header = 'PF ' . $this->creator->first_name . ' ' . $this->creator->last_name . ":   \r\n";

            if (!mb_strpos($text, $header)) {
                $text = $header . $text;
            }
        }

        return $text;
    }

    public function getPlanfixText()
    {
        $re = '/(@\S*)/m';
        $noteText = preg_replace($re, '', $this->text_gitlab);

        if (!Yii::$app->gitlab->isAdmin && $this->origin === 'planfix') {
            $header = 'PF ' . $this->creator->first_name . ' ' . $this->creator->last_name . ':';

            $noteText = str_replace($header, '', $noteText);
        }
        return Yii::$app->gitlab->renderMarkDown($noteText, $this->issue->project->gitlab_id);
    }
}
