<?php

namespace app\core\models;

use app\core\models\AppUser as User;
use yii\db\ActiveRecord;

/**
 * This is the model class for table "core_project".
 *
 * @property int $id
 * @property int $gitlab_id
 * @property int $planfix_id
 * @property string $title
 * @property string $description
 * @property string $gitlab_url
 * @property int $owner_id
 * @property int $lead_developer_id
 * @property User $owner
 */
class Project extends ActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'core_project';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['gitlab_id', 'planfix_id', 'owner_id', 'lead_developer_id'], 'integer'],
            [['description'], 'string'],
            [['title'], 'string', 'max' => 255],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'gitlab_id' => 'Gitlab ID',
            'planfix_id' => 'Planfix ID',
            'title' => 'Название',
            'description' => 'Описание',
            'owner_id' => 'Owner ID',
            'lead_developer_id' => 'Lead Developer ID',
        ];
    }

    public function getOwner()
    {
        return $this->hasOne(User::class, ['id' => 'owner_id']);
    }

    public function getLead()
    {
        return $this->hasOne(User::class, ['id' => 'lead_developer_id']);
    }

    public function getIssues()
    {
        return $this->hasMany(Issue::class, ['project_id' => 'id']);
    }
}
