<?php

namespace app\gitlab\frontend\controllers;

use app\core\models\Project;
use app\gitlab\models\Event;
use app\gitlab\utils\EventParser;
use Yii;
use yii\web\Response;
use yii\web\UnauthorizedHttpException;
use yii\web\UnprocessableEntityHttpException;
use yii\web\Controller;
use app\gitlab\jobs\Event as EventJob;

/**
 * Class WebhookController
 *
 * @package app\gitlab\frontend\controllers
 */
class WebhookController extends Controller
{

    /**
     * @param \yii\base\Action $action
     * @return bool
     * @throws \yii\web\BadRequestHttpException
     */
    public function beforeAction($action)
    {
        $this->enableCsrfValidation = false;
        if (parent::beforeAction($action)) {
            return true;
        }
        return false;
    }

    /**
     * @return array
     * @throws UnauthorizedHttpException
     * @throws UnprocessableEntityHttpException
     */
    public function actionIndex()
    {
        Yii::$app->response->format = Response::FORMAT_JSON;
        $request                    = Yii::$app->request;
        $post                       = $request->post();

        //X-Gitlab-Token
        if (!isset($post['project']) && !isset($post['project_id'])) {
            throw new UnprocessableEntityHttpException('Wrong data');
        }

        $event = new Event();

        if (isset($post['project'])) {
            $project_id = $post['project']['id'];
        } else {
            $project_id = $post['project_id'];
        }

        $project = Project::findOne([
        'gitlab_id' => $project_id,
        'token'     => Yii::$app->request->getHeaders()->get('X-Gitlab-Token'),
        ]);

        if (!$project) {
            throw new UnauthorizedHttpException("Wrong token");
        }

        $event->project_id = $project->id;
        $event->data       = json_encode($post);

        $event->save();

        $id = Yii::$app->queue->push(new EventJob([
            'event_id' => $event->id,
        ]));

        return [
            'success' => true,
            'id' => $id,
        ];
    }

    /**
     * @return bool[]
     * @throws UnauthorizedHttpException
     */
    public function actionRetry()
     {
         if (!YII_ENV_DEV) {
             throw new UnauthorizedHttpException('Allowed on dev envs only');
         }

         $event = Event::find()->orderBy(['id' => SORT_DESC])->one();
         $project = Project::findOne($event->project_id);

         EventParser::parse($event, $project);

         return [
            'success' => true,
         ];
     }
}
