<?php

namespace app\gitlab\models;

use app\core\models\Developer;
use app\core\models\Project;
use app\core\models\User;
use Yii;
use yii\helpers\Url;

/**
 * This is the model class for table "gitlab_commit".
 *
 * @property string $id
 * @property string $title
 * @property string $message
 * @property string $short_id
 * @property int $creator_id
 * @property int $committer_id
 * @property int $project_id
 * @property int $additions_count
 * @property int $deletions_count
 * @property string $created_at
 * @property Project $project
 */
class Commit extends \yii\db\ActiveRecord
{
    /**
     * @inheritdoc
     */
    public static function tableName()
    {
        return 'gitlab_commit';
    }

    /**
     * @inheritdoc
     */
    public function rules()
    {
        return [
            [['id', 'title', 'short_id', 'creator_id', 'committer_id', 'project_id'], 'required'],
            [['message'], 'string'],
            [['creator_id', 'committer_id', 'project_id', 'additions_count', 'deletions_count'], 'integer'],
            [['created_at'], 'safe'],
            [['id', 'title', 'short_id'], 'string', 'max' => 255],
            [['id'], 'unique'],
        ];
    }

    /**
     * @inheritdoc
     */
    public function attributeLabels()
    {
        return [
            'id' => 'ID',
            'title' => 'Название',
            'message' => 'Message',
            'short_id' => 'Short ID',
            'creator_id' => 'Creator ID',
            'committer_id' => 'Committer ID',
            'project_id' => 'Project ID',
            'additions_count' => 'Additions Count',
            'deletions_count' => 'Deletions Count',
            'created_at' => 'Создан',
        ];
    }

    public function getIssues()
    {
        return $this->hasMany(\app\core\models\Issue::class, ['id' => 'issue_id'])
            ->viaTable('gitlab_commit_issue', ['commit_id' => 'id']);
    }

    public function getCommitter()
    {
        return $this->hasOne(User::class, ['id' => 'committer_id']);
    }

    public function getCreator()
    {
        return $this->hasOne(User::class, ['id' => 'creator_id']);
    }

    public function getProject()
    {
        return $this->hasOne(Project::class, ['id' => 'project_id']);
    }

    public function getCommitterUrl()
    {
        return Url::to(['/user/dashboard/index', 'id' => $this->committer_id]);
    }

    public function getCreatorUrl()
    {
        return Url::to(['/user/dashboard/index', 'id' => $this->creator_id]);
    }

    public function getGitlabUrl()
    {
        return sprintf('%s/commit/%s', $this->project->getGitlabUrl(), $this->id);
    }
}
