<?php

namespace app\gitlab\utils;

use Yii;
use app\planfix\classes\PFApi;
use Exception;

class EventParser
{
    public static function parse($event, $project)
    {
        
        $data = json_decode($event->data);

        PFApi::checkProject($project);

        if ($data->object_kind == 'issue') {
            static::parseIssue($event, $data);
        }

        if ($data->object_kind == 'note') {
            static::parseNote($event, $data);
        }

        if ($data->object_kind == 'push') {
            static::parsePush($event, $data);
        }

        // Yii::$app->gitlab->syncMissing();
    }

    private static function parseIssue($event, $data)
    {
        if (property_exists($data, 'object_attributes') && count((array)$data->changes)) {
            // $needNotes = (property_exists($data, 'changes') && (property_exists($data->changes, 'total_time_spent')) || property_exists($data->changes, 'time_estimate')) ? true: false;
            $needNotes = true;
            static::syncIssue($data->object_attributes->iid, $event->project_id, $needNotes);
        }
    }

    private static function parseNote($event, $data)
    {
        if (property_exists($data, 'issue')) {
            Yii::$app->gitlab->importNote($data->object_attributes->id, $data->issue->iid, $event->project_id);
        }
    }

    private static function parsePush($event, $data)
    {
        if (property_exists($data, 'commits')) {
            foreach ($data->commits as $commit) {
                preg_match_all('/#[0-9]{1,}/', $commit->message, $matches);

                if (isset($matches[0])) {
                    foreach ($matches[0] as $task) {
                        try {
                            static::syncIssue(substr($task, 1), $event->project_id);
                        } catch (Exception $e) {
                        }
                    }
                }
            }
        }
    }

    public static function syncIssue($issueId, $projectId, $needNotes = false)
    {
        return Yii::$app->gitlab->importIssue($issueId, $projectId, $needNotes);
    }

    public static function syncCommits($project)
    {
        return Yii::$app->gitlab->importCommits($project);
    }

    public static function syncProjectMembers($project)
    {
        return Yii::$app->gitlab->importProjectMembers($project);
    }

}
