<?php

namespace app\gitlab\utils;

use Http\Client\Common\Plugin;
use Http\Client\Exception;
use Psr\Http\Message\RequestInterface;
use Psr\Http\Message\ResponseInterface;
use app\core\models\RequestLog;

/**
 * Record HTTP calls.
 */
final class RequestLogger implements Plugin
{

    /**
     * {@inheritdoc}
     */
    public function handleRequest(RequestInterface $request, callable $next, callable $first)
    {

        return $next($request)->then(function (ResponseInterface $response) use ($request) {
            $this->log();
            return $response;
        }, function (Exception $exception) use ($request) {
            $this->log();
            throw $exception;
        });
    }

    public function log()
    {
        $log = RequestLog::find()->where([
            'date' => date('Y-m-d'),
            'service' => 'gitlab'
        ])->one();

        if (!$log) {
            $log = new RequestLog();
            $log->service = 'gitlab';
            $log->date = date('Y-m-d');
            $log->count = 0;
        }

        $log->count += 1;
        $log->save();
    }
}
