<?php

namespace app\planfix\components;

use Exception;
use SimpleXMLElement;
use Yii;
use yii\base\Component;
use yii\helpers\ArrayHelper;
use app\core\models\Planfix;
use app\planfix\helpers\ApiHelper;
use app\core\models\RequestLog;

/**
 * Create magic with Planfix
 */
class PlanfixApi extends Component
{
    /**
     * Account name <account>.planfix.ru
     *
     * Set just account name
     */
    public $account = '';

    /**
     * Api Key
     */
    public $api_key = '';

    /**
     * Api Secret
     */
    public $api_secret = '';

    /**
     * Api Server
     *
     * mb this: https://api.planfix.ru/xml/
     * or this: https://apiru.planfix.ru/xml/
     */
    public $api_server = '';

    public $tpl_dev_id = null;
    public $pr_field_lead_dev = null;
    public $time_analytic_id = null;
    public $time_analytic_date_id = null;
    public $time_analytic_time_id = null;
    public $time_analytic_user_id = null;
    public $process_id = null;
    public $default_status_id = null;

    public $curlInfo = null;


    public function init()
    {
        parent::init();

        $planfix = Planfix::find()->one();
        $this->account = $planfix? $planfix->account : '';
        $this->api_key = $planfix? $planfix->api_key : '';
        $this->api_secret = $planfix? $planfix->api_secret : '';
        $this->api_server = $planfix? $planfix->api_server : 'https://api.planfix.ru/xml';

        $this->tpl_dev_id = $planfix? $planfix->tpl_dev_id : '';
        $this->pr_field_lead_dev = $planfix? $planfix->pr_field_lead_dev : '';
        $this->time_analytic_id = $planfix? $planfix->time_analytic_id : '';
        $this->time_analytic_date_id = $planfix? $planfix->time_analytic_date_id : '';
        $this->time_analytic_time_id = $planfix? $planfix->time_analytic_time_id : '';
        $this->time_analytic_user_id = $planfix? $planfix->time_analytic_user_id : '';
        $this->process_id = $planfix? $planfix->process_id : '';
        $this->default_status_id = $planfix? $planfix->default_status_id: '';
    }

    /**
     * Total requests
     */
    public $requests_count = 0;

    /**
     * Error messages list
     */
    public $errorCodes = [
        "0001"  => "Неверный API Key",
        "0002"  => "Приложение заблокировано",
        "0003"  => "Ошибка XML разбора. Некорректный XML",
        "0004"  => "Неизвестный аккаунт",
        "0005"  => "Ключ сессии недействителен (время жизни сессии истекло)",
        "0006"  => "Неверная подпись",
        "0007"  => "Превышен лимит использования ресурсов (ограничения, связанные с лицензиями или с количеством запросов)",
        "0008"  => "Неизвестное имя функции",
        "0009"  => "Отсутствует один из обязательных параметров функции",
        "0010"  => "Аккаунт заморожен",
        "0011"  => "На площадке аккаунта производится обновление программного обеспечения",
        "0012"  => "Отсутствует сессия, не передан параметр сессии в запрос",
        "0013"  => "Неопределенный пользователь",
        "0014"  => "Пользователь неактивен",
        "0015"  => "Недопустимое значение параметра",
        "0016"  => "В данном контексте параметр не может принимать переданное значение",
        "0017"  => "Отсутствует значение для зависящего параметра",
        "0018"  => "Функции/функционал не реализована",
        "0019"  => "Заданы конфликтующие между собой параметры",
        "0020"  => "Вызов функции запрещен",
        "0021"  => "Запрошенное количество объектов больше максимально разрешенного для данной функции",
        "0022"  => "Использование API недоступно для бесплатного аккаунта",
        "0023"  => "Запрошенное действие невозможно в рамках текущего тарифного плана аккаунта",
        "0024"  => "Аккаунт расположен в другом датацентре, проверьте url запроса",
        "1001"  => "Неверный логин или пароль",
        "1002"  => "На выполнение данного запроса отсутствуют права (привилегии)",
        "2001"  => "Запрошенный проект не существует",
        "2002"  => "На выполнение данного запроса отсутствуют права (привилегии)",
        "2003"  => "Ошибка добавления проекта",
        "3001"  => "Указанная задача не существует",
        "3002"  => "Нет доступа к над задаче",
        "3003"  => "Проект, в рамках которого создается задача, не существует",
        "3004"  => "Проект, в рамках которого создается задача, не доступен",
        "3005"  => "Ошибка добавления задачи",
        "3006"  => "Время \"Приступить к работе\" не может быть больше времени \"Закончить работу до\"",
        "3007"  => "Неопределенная периодичность, скорее всего задано несколько узлов, которые конфликтуют друг с другом или не указан ни один",
        "3008"  => "Нет доступа к задаче",
        "3009"  => "Нет доступа на изменение данных задачи",
        "3010"  => "Данную задачу отклонить нельзя (скорее всего, она уже принята этим пользователем)",
        "3011"  => "Данную задачу принять нельзя (скорее всего, она уже принята этим пользователем)",
        "3012"  => "Пользователь, выполняющий запрос, не является исполнителем задачи",
        "3013"  => "Задача не принята (для выполнения данной функции задача должна быть принята)",
        "3014"  => "Задача в процессе изменения (сценарием или другим API запросом). Повторите запрос через некоторое время.",
        "4001"  => "На выполнение данного запроса отсутствуют права (привилегии)",
        "4002"  => "Действие не существует",
        "4003"  => "Ошибка добавления действия",
        "4004"  => "Ошибка обновления данных",
        "4005"  => "Ошибка обновления данных",
        "4006"  => "Попытка изменить статус на недозволенный",
        "4007"  => "В данном действии запрещено менять статус",
        "4008"  => "Доступ к комментария/действию отсутствует",
        "4009"  => "Доступ к задаче отсутствует",
        "4010"  => "Указанная аналитика не существует",
        "4011"  => "Для аналитики были переданы не все поля",
        "4012"  => "Указан несуществующий параметр для аналитики",
        "4013"  => "Переданные данные не соответствуют типу поля",
        "4014"  => "Указанный ключ справочника нельзя использовать",
        "4015"  => "Указанный ключ справочника не существует",
        "4016"  => "Указанный ключ данных поля не принадлежит указанной аналитике",
        "5001"  => "Указанная группа пользователей не существует",
        "5002"  => "На выполнение данного запроса отсутствуют права (привилегии)",
        "5003"  => "Ошибка добавления",
        "6001"  => "На выполнение данного запроса отсутствуют права (привилегии)",
        "6002"  => "Данный e-mail уже используется",
        "6003"  => "Ошибка добавления сотрудника",
        "6004"  => "Пользователь не существует",
        "6005"  => "Ошибка обновления данных",
        "6006"  => "Указан идентификатор несуществующей группы пользователей",
        "7001"  => "На выполнение данного запроса отсутствуют права (привилегии)",
        "7002"  => "Клиент не существует",
        "7003"  => "Ошибка добавления клиента",
        "7004"  => "Ошибка обновления данных",
        "8001"  => "На выполнение данного запроса отсутствуют права (привилегии)",
        "8002"  => "Контакт не существует",
        "8003"  => "Ошибка добавления контакта",
        "8004"  => "Ошибка обновления данных",
        "8005"  => "Контакт не активировал доступ в ПланФикс",
        "8006"  => "Контакту не предоставлен доступ в ПланФикс",
        "8007"  => "E-mail, указанный для логина, не уникален",
        "8008"  => "Попытка установки пароля для контакта, не активировавшего доступ в ПланФикс",
        "8009"  => "Ошибка обновления данных для входа в систему",
        "9001"  => "На выполнение данного запроса отсутствуют права (привилегии)",
        "9002"  => "Запрашиваемый файл не существует",
        "9003"  => "Ошибка загрузки файла",
        "9004"  => "Попытка загрузить пустой список файлов",
        "9005"  => "Недопустимый символ в имени файла",
        "9006"  => "Имя файла не уникально",
        "9007"  => "Ошибка файловой системы",
        "9008"  => "Ошибка возникает при попытке добавить файл из проекта для проекта",
        "9009"  => "Файл, который пытаются добавить к задаче, является файлом другого проекта",
        "10001" => "На выполнение данного запроса отсутствуют права (привилегии)",
        "10002" => "Аналитика не существует",
        "10003" => "Переданный параметр группы аналитики не существует",
        "10004" => "Переданный параметр справочника аналитики не существует",
        "11001" => "Указанной подписки не существует",
    ];

    /**
     * Suffixes for result in results ;D
     */
    public $suffixes = [
        'project'       => 'projects',
        'task'          => 'tasks',
        'file'          => 'files',
        'taskStatusSet' => 'taskStatusSets',
        'taskStatus'    => 'taskStatuses',
        'action'        => 'actions',
        'user'          => 'users',
        'contact'       => 'contacts',
    ];

    /**
     * Default result for requests
     */
    public $defaultResultList = [
        'list'       => [],
        'totalCount' => 0,
        'count'      => 0,
    ];

    public $canSilentsMethods = [
        'task.update',
    ];

    /**
     * Initializes a Planfix Client
     *
     * Required parameters:
     *  - account - Account name <account>.planfix.ru
     *  - api_key
     *  - api_secret
     *  - api_server - https://apiru.planfix.ru/xml/ or https://api.planfix.ru/xml/
     *
     * @param array $config The array containing required parameters
     */
    // public function __construct($config = [])
    // {
    //     set_time_limit(0);

    //     extract(array_merge([
    //         'account'    => '',
    //         'api_key'    => '',
    //         'api_secret' => '',
    //         'api_server' => '',
    //     ], $config));

    //     $this->account    = $account;
    //     $this->api_key    = $api_key;
    //     $this->api_secret = $api_secret;
    //     $this->api_server = $api_server;
    // }
    //


    public function exec($requestXml, $count = 0)
    {
        $ch = curl_init($this->api_server);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1); // не выводи ответ на stdout
        curl_setopt($ch, CURLOPT_HEADER, 1); // получаем заголовки
        curl_setopt($ch, CURLOPT_TIMEOUT, 40); // устанавливам максимальное время ожидания
        // не проверять SSL сертификат
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        // не проверять Host SSL сертификата
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);

        curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
        curl_setopt($ch, CURLOPT_USERPWD, $this->api_key . ':' . $this->api_secret);

        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $requestXml->asXML());

        $response     = curl_exec($ch);
        $error        = curl_error($ch);
        $header_size  = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
        $responseBody = substr($response, $header_size);
        $this->curlInfo = curl_getinfo($ch);

        curl_close($ch);

        if ((strpos($error, 'Operation timed out after') !== false || $this->curlInfo['http_code'] === 0) && $count <= 5) {
            ++$count;
            sleep(10);
            $responseBody = $this->exec($requestXml, $count);
        }

        $this->requests_count++;

        

        return simplexml_load_string($responseBody);
    }

    /**
     * Create XML request
     *
     * @return SimpleXMLElement the XML request
     */
    public function createXml($method)
    {
        $str_xml = '<?xml version="1.0" encoding="UTF-8"?><request method="' . $method . '"></request>';

        $requestXml          = new SimpleXMLExtended($str_xml);
        $requestXml->account = $this->account;

        if (in_array($method, $this->canSilentsMethods)) {
            $requestXml->silent = '1';
        }

        return $requestXml;
    }

    public function checkSend($objectName, $result)
    {
        $success = true;

        if (!array_key_exists($objectName, $result)) {
            $success = false;
        }

        if (array_key_exists($objectName, $result)) {
            if (0 === (int) $result[$objectName]['@attributes']['totalCount']) {
                $success = false;
            }
        }

        return $success;
    }

    public function removeFields($list, $options)
    {
        if (array_key_exists('removeFields', $options)) {
            foreach ($list as $itemKey => $itemValue) {
                foreach ($itemValue as $propertyKey => $propertyValue) {
                    if (in_array($propertyKey, $options['removeFields'])) {
                        unset($list[$itemKey][$propertyKey]);
                    }
                }
            }
        }

        return $list;
    }

    public function convertObjectToArray($object = [])
    {
        if (array_key_exists('id', $object)) {
            $object = [$object];
        }

        return $object;
    }

    public function getCountPages($data)
    {
        return (int) floor(ceil($data['totalCount'] / $data['count']));
    }

    public function setIdsToItems($items = [])
    {
        $newItems = [];

        foreach ($items as $item) {
            if (array_key_exists('id', $item)) {
                $newItems[$item['id']] = $item;
            }
        }

        return $newItems;
    }

    /**
     * Import parameters to XML request
     *
     * @param SimpleXMLElement The XML request
     * @param array Parameters
     *
     * @return SimpleXMLElement the XML request
     */
    public function importOptions($xml, $options)
    {
        $textFields = [
            'description',
            'title',
            'post',
        ];

        foreach ($options as $key => $val) {
            if (is_array($val)) {
                if (is_int($key)) {
                    $this->importOptions($xml, $val);
                } else {
                    $newXml = $xml->addChild($key);
                    foreach ($val as $key2 => $val2) {
                        if (!is_null($val2)) {
                            if (is_array($val2)) {
                                $this->importOptions($newXml, $val2);
                            } else {
                                if (in_array($key2, $textFields)) {
                                    $node_note = $newXml->addChild($key2);
                                    $node_note->addCData($val2);
                                } else {
                                    $newXml->addChild($key2, $val2);
                                }
                            }
                        }
                    }
                }
            } else {
                if (!is_null($val)) {
                    if (in_array($key, $textFields)) {
                        $node_note = $xml->addChild($key);
                        $node_note->addCData($val);
                    } else {
                        $xml->addChild($key, $val);
                    }
                }
            }
        }

        return $xml;
    }

    public function array2xml($array, $xml = false)
    {
        if ($xml === false) {
            $xml = new SimpleXMLElement('<root/>');
        }
        foreach ($array as $key => $value) {
            if (is_array($value)) {
                if (is_numeric($key)) {
                    $this->array2xml($value, $xml);
                } else {
                    $this->array2xml($value, $xml->addChild($key));
                }
            } else {
                if (in_array($key, [
                    'description',
                    'title',
                    'post',
                ])) {
                    $node_note = $xml->addChild($key);
                    $node_note->addCData($value);
                } else {
                    $xml->addChild($key, $value);
                }
            }
        }
        return $xml;
    }

    public function logRequest($method)
    {
        $log = RequestLog::find()->where([
            'date' => date('Y-m-d'),
            'service' => 'planfix'
        ])->one();

        if (!$log) {
            $log = new RequestLog();
            $log->service = 'planfix';
            $log->count = 0;
            $log->date = date('Y-m-d');
        }

        $log->count += 1;
        $log->save();

        Yii::$app->impulse->air('request_log', $method);
    }

    public function api($method, $params = [], $inObject = false, $array2xml = false)
    {
        $xml = $this->createXml($method);

        if ($inObject) {
            $object = $xml->addChild(explode('.', $method)[0]);
            $this->importOptions($object, $params);
        } else {
            if ($array2xml) {
                $this->array2xml($params, $xml);
            } else {
                $this->importOptions($xml, $params);
            }
        }


        if (YII_ENV_DEV) {
            Yii::$app->impulse->air('debug', print_r($xml->asXML(), true));
        }


        $result = json_decode(json_encode($this->exec($xml)), true) ?? [
            'code' => 'Что-то пошло не так',
        ];

        $this->logRequest($method);

        if (!is_array($result)) {
            throw new Exception('Request failed ' . print_r($this->curlInfo, true));
        }
        // If in result exists code
        if (array_key_exists('code', $result)) {
            // var_export("requestXml -> ");
            // var_export($xml);
            // var_export("asXML -> ");
            // var_export($xml->asXML());
            // var_export("result -> ");
            // var_export($result);
            // var_export("error -> ");
            throw new Exception($this->errorCodes[$result['code']] . ' -> ' . print_r($result['message'], true));
        }

        // $item = $this->removeFields($result[$objectName], $options);

        // $methodName = explode('.', $method)[0];

        // if (array_key_exists($methodName, $this->suffixes)) {
        //     $suffixMethod = $this->suffixes[$methodName];

        //     if (array_key_exists($suffixMethod, $result)) {
        //         if (array_key_exists($methodName, $result[$suffixMethod])) {
        //             $item = $result[$suffixMethod][$methodName];

        //             return $item;
        //         }
        //     }
        // }

        return $result;
    }

    /**
     * Добавление комментария/действия в задачу планфикса
     *
     * @param array $options
     *
     * @return array|mixed
     * @throws Exception
     */
    public function addActionToTask($options = [])
    {
        /**
         * @var $task_id      int
         * @var $owner_id     int
         * @var $comment      string
         * @var $notify_users array
         * @var $analytics    array
         */
        extract(
            ArrayHelper::merge([
                'task_id'      => null,
                'owner_id'     => null,
                'comment'      => null,
                'notify_users' => [],
                'analytics'    => [],
            ], $options)
        );

        return $this->api('action.add', [
            [
                'action' => ArrayHelper::merge([['analitics' => $analytics]], [
                    'description' => $comment,
                    [
                        'task' => [
                            'id' => $task_id,
                        ],
                    ],
                    [
                        'owner' => [
                            'id' => $owner_id,
                        ],
                    ],
                    [
                        'notifiedList' => [
                            [
                                'user' => array_map(function ($user_id) {
                                    return ['id' => $user_id];
                                }, $notify_users),
                            ],
                        ],
                    ],
                ]),
            ],
        ]);
    }


    public function getTemplates()
    {

        try {
            $templates = ApiHelper::paginate('task.getList', [
                'target' => 'template',
            ], 'tasks', 'task');
        } catch (\Exception $e) {
            $templates = [];
        }

        $data = [];

        foreach ($templates as $template) {
            $data[$template['id']] = $template['title'];
        }

        return $data;
    }

    public function getProjectFields()
    {
        try {
            $projects = ApiHelper::paginate('project.getList', [
                'target' => 'all',
            ], 'projects', 'project');
        } catch (\Exception $e) {
            $projects = [];
        }

        $data = [];

        foreach ($projects as $project) {
            if (isset($project['customData']['customValue'])) {
                if (isset($project['customData']['customValue']['field'])) {
                    $project['customData']['customValue'] = [ $project['customData']['customValue'] ];
                }
                foreach ($project['customData']['customValue'] as $customValue) {
                    $data[$customValue['field']['id']] = $customValue['field']['name'];
                }
            }
        }

        return $data;
    }

    public function getProcessesList()
    {
        $data = [];

        $taskProcesses = Yii::$app->planfix->api('taskStatus.getSetList');

        $sets = $taskProcesses['taskStatusSets']['taskStatusSet'];
        if (!isset($sets[0])) {
            $sets = [$sets];
        }

        foreach ($sets as $set) {
            $data[$set['id']] = $set['name'];
        }

        return $data;
    }

    
    public function getAnalytics()
    {
        try {
            $analytics = $this->api('analitic.getList');
        } catch (\Exception $e) {
            $analytics = [
                'analitics' => [
                    'analitic' => [],
                ]
            ];
        }

        $data = [];

        if (isset($analytics['analitics']['analitic'])) {
            if (isset($analytics['analitics']['analitic']['id'])) {
                $analytics['analitics']['analitic'] = [ $analytics['analitics']['analitic'] ];
            }
            foreach ($analytics['analitics']['analitic'] as $analitic) {
                $data[$analitic['id']] = $analitic['name'];
            }
        }

        return $data;
    }

    public function getAnalyticFields($id)
    {
        try {
            $options = $this->api('analitic.getOptions', [
                'analitic' => [
                    'id' => $id
                ]
            ]);
        } catch (\Exception $e) {
            $options = [
                'analitic' => [
                    'fields' => [
                        'field' => []
                    ]
                ]
            ];
        }

        $data = [];

        if (isset($options['analitic']['fields']['field'])) {
            if (isset($options['analitic']['fields']['field']['id'])) {
                $options['analitic']['fields']['field'] = [ $options['analitic']['fields']['field'] ];
            }
            foreach ($options['analitic']['fields']['field'] as $field) {
                $data[$field['id']] = $field['name'];
            }
        }

        return $data;
    }
}


class SimpleXMLExtended extends SimpleXMLElement
{
    public function addCData($cdata_text)
    {
        $node = dom_import_simplexml($this);
        $no   = $node->ownerDocument;
        $node->appendChild($no->createCDATASection($cdata_text));
    }
}
