<?php

namespace app\planfix\helpers;

use app\planfix\models\PlanfixProject;
use app\planfix\models\PlanfixUser;
use app\user\models\Profile;
use Exception;
use Yii;

class ApiHelper
{


    public static function getTemplates()
    {
        return static::paginate('task.getList', ['target' => 'template']);
    }

    public static function getProjects()
    {
        return static::paginate('project.getList', [], 'projects', 'project');
    }

    public static function getUsers()
    {
        return static::paginate('user.getList', [], 'users', 'user');
    }

    public static function getContacts()
    {
        return static::paginate('contact.getList', [], 'contacts', 'contact');
    }

    public static function getTasks($projectId)
    {
        return static::paginate('task.getList', [
            'project' => [
                'id' => $projectId,
            ]
        ]);
    }

    public static function paginate($method, $options, $attribute = 'tasks', $item = 'task', $pageSize = 100)
    {
        $results = [];

        $options['pageCurrent'] = 0;
        $count                  = Yii::$app->planfix->api($method, $options);

        $count = $count[$attribute]['@attributes']['totalCount'];

        for ($i = 1; $i <= floor($count / $pageSize) + 1; $i++) {
            $options['pageCurrent'] = $i;
            $options['pageSize']    = $pageSize;
            $data                   = Yii::$app->planfix->api($method, $options);

            if (isset($data[$attribute][$item])) {
                $results = array_merge($results, $data[$attribute][$item]);
            } else {
                $results = array_merge($results, [$data[$attribute]]);
            }
        }
        if (count($results) && !isset($results[0])) {
            $results = [ $results ];
        }

        return $results;
    }

    public static function syncProjects()
    {
        $projects = static::getProjects();

        foreach ($projects as $project) {
            $planfixProject = PlanfixProject::findOne([
                'id' => $project['id'],
            ]);

            if (!$planfixProject) {
                $planfixProject     = new PlanfixProject();
                $planfixProject->id = $project['id'];
            }

            $planfixProject->name        = $project['title'];
            $planfixProject->description = is_array($project['description']) ? '' : $project['description'];
            $planfixProject->owner_id    = isset($project['owner']) ? $project['owner']['id'] : 0;
            $planfixProject->client_id   = isset($project['client']) ? $project['client']['id'] : 0;

            $save = $planfixProject->save();

            if (!$save) {
                throw new Exception(print_r($planfixProject->getErrors(), true));
            }
        }
    }

    public static function syncUsers()
    {
        $users = static::getUsers();

        foreach ($users as $user) {
            $planfixUser = PlanfixUser::findOne([
                'id' => $user['id'],
            ]);

            if (!$planfixUser) {
                $planfixUser     = new PlanfixUser();
                $planfixUser->id = $user['id'];
            }

            $planfixUser->name     = $user['name'];
            $planfixUser->lastName = is_array($user['lastName']) ? '' : $user['lastName'];
            $planfixUser->email    = is_array($user['email']) ? '' : $user['email'];

            $save = $planfixUser->save();

            if (!$save) {
                throw new Exception(print_r($planfixUser->getErrors(), true));
            }

            $userProfileQuery = Profile::find()->where([
                'first_name' => $planfixUser->name,
                'last_name'  => $planfixUser->lastName,
            ]);

            if ($userProfileQuery->count() == 1) {
                $user             = $userProfileQuery->one()->user;
                $user->planfix_id = $planfixUser->id;
                $user->save();
            }
        }
    }

    /**
     * Проверка на наличие статуса
     *
     * @param $data
     *
     * @return bool
     */
    public static function statusIsOk($data)
    {
        if (array_key_exists('@attributes', $data)) {
            if (array_key_exists('status', $data['@attributes'])) {
                if ($data['@attributes']['status'] === 'ok') {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Если объект не в массиве, обернем объект в массив
     *
     * @param        $object
     * @param string $checkKey
     *
     * @return array
     */
    public static function convertObjectToArray($object, $checkKey = 'id')
    {
        if (array_key_exists($checkKey, $object)) {
            $object = [$object];
        }

        return $object;
    }

    /**
     * Добавить всем элементам массива, значение ключа, равное значению нужного ключа внутри элемента
     *
     * @param $array
     * @param $key
     *
     * @return array
     */
    public static function addKeyFromValueInArray($array, $key)
    {
        $newArray = [];

        foreach ($array as $item) {
            $newArray[$item[$key]] = $item;
        }

        return $newArray;
    }

    public static function findCustomDataProperty($id, $data)
    {
        if (isset($data['customData']['customValue'])) {
            foreach ($data['customData']['customValue'] as $customValue) {
                if (isset($customValue['field']) && $customValue['field']['id'] == $id) {
                    return [
                        'value' => (is_array($customValue['value'])? null : $customValue['value']),
                        'text' => (is_array($customValue['text'])? null :$customValue['text']),
                    ];
                }
            }
        }

        return [
            'value' => null,
            'text' => null,
        ];
    }
}
